# Lab 01: Fuzzing with AFL

TL;DR The goal of this lab is to get familiar with fuzzing. You will set up AFL
on your machine and experiment with different modes and forms of fuzzing. In a
total of three tasks (one optional) you will play with a fuzzing engine using
different forms of coverage tracking: through afl-gcc, qemu, and by using
LLVM-based instrumentation.

Please collect and write up the results for each task in a short report.
For each task you should invest about 2 hours of thinking time and 4-6 hours of
CPU time. Let the CPU do the work. Send your (short) report to
<mailto:mathias.payer@epfl.ch> by November 13 (in 3 weeks).


## Background: AFL and LAVA-M

**AFL** is a coverage-guided greybox fuzzer. AFL does not require an input
specification to generate new fuzzing inputs but leverages coverage as an
indication of input quality. Based on a set of inputs, AFL randomly mutates the
input. Whenever new (or different) coverage is reached, AFL adds the input to
the set of fuzzing inputs. To collect coverage information, AFL uses a simple
instrumentation to track edges hashed into a coverage map. You'll find AFL at
<http://lcamtuf.coredump.cx/afl/>.

**LAVA-M** is a set of benchmarks taken from coreutils with injected synthetic bugs.
The advantage of LAVA-M is that any input that triggers an injected bug
terminates the application with a message that the bug was triggered. This makes
triaging very simple. For this first lab, we'll be using LAVA-M to test
different fuzzers and compare them against each other. The LAVA-M benchmarks can
be found at <http://panda.moyix.net/~moyix/lava_corpus.tar.xz> with the paper at
<https://seclab.ccs.neu.edu/static/publications/sp2016lava.pdf>.

For your convenience, just execute the `getfiles.sh` script to download both AFL
and LAVA-M. If you chose not to do task 3, you don't need the LAVA-M files.


## Task 0: set-up your fuzzing environment

The first step is to set up the necessary build environment. You'll need at
least AFL installed. For the optional task, you'll also need to build your own
LAVA-M binaries. 

* **Download the lab files.** You'll need to download and install the lab files
  with the sample binaries. The lab has been tested on Debian and Ubuntu but
  should equally work on MacOS. If you're struggling with the environment, it
  may be easiest to download and work on a Debian VM. Pick your favorite or use
  Docker (e.g., untested:
  <https://medium.com/@hudsonmendes/docker-have-a-ubuntu-development-machine-within-seconds-from-windows-or-mac-fd2f30a338e4>).

* **Install AFL.** Download the AFL tarball. The link is at the bottom of the AFL
  page. Read the introduction (at least `QuickStartGuide.txt` to install it).
  You'll need a basic build environment to get AFL up and running. Follow the
  build instructions for AFL (aka `make`).
  For task 2, you will need to enable QEMU support. Make sure to build qemu
  support by executing `buikd_qemu_support.sh` in `qemu_mode`.
  You'll be required to run some processes for a couple of hours. Most laptops
  and Desktops will work fine. If you're running on an SSD, you may consider
  running your fuzzer in a RAM disk:
  <http://www.cipherdyne.org/blog/2014/12/ram-disks-and-saving-your-ssd-from-afl-fuzzing.html>.

* For the optimal task 3, **Download LAVA-M.** Download the LAVA-M tarball. The
  link is on the LAVA-M site. You may need a bunch of additional packages to get
  LAVA-M to run, e.g., libacl1-dev, libselinux1-dev, libcap-dev, and libgmp-dev
  on Debian or Ubuntu.
 

## Task 1: fuzzing an instrumented binary

In this first task, we will fuzz two binaries with AFL to find bugs: `base64`
and `md5sum`, both part of the coreutils.  Luckily, `base64-afl` and
`md5sum-afl` have already been instrumented and compiled with afl-gcc. afl-gcc
is one of three ways to instrument binaries with an afl coverage tracking. 

Read the AFL documentation to figure out the best configuration. Include your
command line in the report and discuss why you chose individual settings. Note
that `base64` must run with the `-d` option and `md5sum` must run with the `-c`
option.

The goal of this task is to study the effects of different input seeds. For each
configuration below, measure (i) how long it takes until you find the first
crash and (ii) how many crashes you find in one hour of fuzzing time.

* Fuzz the programs with a 'fuzz' seed (use a file with 'fuzz' as content as
  sole input).

* Think about the two programs and come up with "reasonable" input seeds. 
  In this subtask, you are trying to "jump start" the fuzzer by preparing
  starting input seeds to guide the fuzzer along more reasonable paths.
  Think valid inputs with different formats. Discuss your strategy in the
  report.


## Task 2: fuzzing an uninstrumented binary

If the source code is not available, then AFL can use a binary translation
engine (QEMU) to dynamically insert coverage tracking into the executed binary.
Repeat the same fuzzing campaign from Task 1 by adding "`-Q`" to the fuzzer
invocation.

* What is the performance difference between running an instrumented binary and
  an uninstrumented binary?

* What is the source of this overhead?


## Task 3: instrumenting a binary (optional)

LLVM-AFL allows you to optimize the binary through a compiler-based toolchain.
Instead of inserting the AFL instrumentation late in the compilation (as afl-gcc
does) or at runtime (as QEMU mode does), LLVM-AFL inserts the instrumentation
early during compilation. Additionally, the programmer can leverage a fork
server that initialized the binary up to a certain point and starts fuzzing only
from that point in time.

Build the LAVA-M benchmarks `base64` and `md5sum` using LLVM-AFL. Look at the
source files and consider where to insert the AFL fork server and how to speed
up fuzzing.

Repeat the same fuzzing campaign from Task 1 with the LLVM-AFL binaries.

* What is the performance difference between running the LLVM instrumentation
  and the GCC instrumentation?

* What is the source of this speedup?

* What else would you do to speed up fuzzing execution? (This is an open
  question)
